﻿using UnityEngine;
using System.Collections;
using System;
using System.IO.Ports;
using System.Threading;
using System.Collections.Generic;
using System.Collections;

public class SerialHandler : MonoBehaviour
{
	public delegate void SerialDataReceivedEventHandler(string message);
	public event SerialDataReceivedEventHandler OnDataReceived;

	public string portName = "/dev/tty.usbserial-FT9KZ1P0";
	///dev/tty.usbserial-FTFMEYH7
	public int baudRate    = 115200;
	
	private SerialPort serialPort_;
	private Thread thread_;
	private bool isRunning_ =  false;  
	 
	private string message_;

	private int byte2=0;
	private int byte3=0;
	private int byte4=0;
	private int byte1=0;

	private double nsamples=100.0;
	private double low = 0;
	private double med = 0;
	private double high = 0;
	private double value = 0;

	private List<Vector3> spherePos = new List<Vector3> ();
	private List<Vector3> cubePos = new List<Vector3> ();
	private int sphereCount = 0;
	private int cubeCount = 0;
	private int voxelCount = 0;

	[SerializeField]
	private GameObject voxelPrefab;

	private bool isNewMessageReceived_ = false;

	private Dictionary<string, CustomPrefab1> voxel = new Dictionary<string, CustomPrefab1>();
	
	void Awake()
	{
		Open();      
	}
	void Start()
	{
		
		Vector3 center = new Vector3 (0, 0, 0);

		Vector3 tempScale = new Vector3 (1.0f, 1.0f, 1.0f);



		float radius=8;
		for (float i = -radius; i < radius; i++) {
			for (float j = -radius; j < radius; j++) {
				for (float k = -radius; k < radius; k++) {
					Vector3 tempPos = new Vector3 (i, j, k)+center;
					cubePos.Add (tempPos);
					createPrefab (voxel, voxelCount, tempPos, tempScale, "Sphere", voxelPrefab);
					voxelCount++;
					cubeCount++;
				}
			}
		}

		radius=10;
		for (float i = -radius; i < radius; i++) {
			for (float j = -radius; j < radius; j++) {
				for (float k = -radius; k < radius; k++) {
					Vector3 tempPos = new Vector3 (i, j, k)+center;
					float distance = Vector3.Distance (center, tempPos);
					if(distance <= radius && sphereCount>=cubeCount)
					{
						//cubePos.Add(center);
					}
					else if (distance <= radius) 
					{
						spherePos.Add (tempPos);
						voxelCount++;
						sphereCount++;
					}
				}
			}
		}

		Debug.Log (spherePos.Count+","+cubePos.Count);

	}
	
	void Update()
	{  
		for (int i = 0; i < cubeCount; i++) {
			GameObject tempGO = GameObject.Find ("Sphere" + i);
			double mappedX = map (value, 0.0, 1000.0, cubePos [i].x, spherePos [i].x);
			double mappedY = map (value, 0.0, 1000.0, cubePos [i].y, spherePos [i].y);
			double mappedZ = map (value, 0.0, 1000.0, cubePos [i].z, spherePos [i].z);

			tempGO.transform.position = new Vector3 ((float)mappedX, (float)mappedY, (float)mappedZ);
		}
		
		if(Input.GetKey(KeyCode.LeftArrow)){
			 
		}
		if (isNewMessageReceived_) {
 			//OnDataReceived(message_);
 			//OnDataReceived(total1.ToString("g") + "a"+ total2.ToString("g") +"a"+ total3 .ToString("g"));
		}        
	}
	
	void OnDestroy()
	{
		Close();
	}
	 
	private void Open()
	{
		Debug.Log ("here");
        string[] portNames;
        portNames = System.IO.Ports.SerialPort.GetPortNames();
        for(int i=0;i< portNames.Length; ++i)
        {
            Debug.Log(portNames[i]);
        }

        //if (portNames.Length > 0)
            //return portNames[0];
        //else
            //return "COM3";

        serialPort_ = new SerialPort(portName, baudRate, Parity.None, 8, StopBits.One);
   		serialPort_.Open();   
		Debug.Log("open the port ");
		isRunning_ = true; 
		
		thread_ = new Thread(Read);
		thread_.Start();
	}
	 
	private void Close() 
	{
		isRunning_ = false;
		
		if (thread_ != null && thread_.IsAlive) {
			thread_.Join();
		}
		
		if (serialPort_ != null && serialPort_.IsOpen) {
			serialPort_.Close();
			serialPort_.Dispose();
		}
	}
	
	private void Read()
	{
		//while (isRunning_ && serialPort_ != null && serialPort_.IsOpen) {
        while ( serialPort_.IsOpen)
            {
                try {
                //Debug.Log(serialPort_.ReadExisting());
                //Debug.Log("hdhd");
                if (serialPort_.BytesToRead > 0) {

					//Debug.Log("value");
					Debug.Log("data is coming" + byte1 + ":"+ byte2 + ":"+ byte3 + ":"+ byte4 );

					if(byte1==48){
						value=0.0;
						Debug.Log("0");
					}else{
						value=1000.0;
						Debug.Log("1");
					}
					byte1=byte2;
					byte2=byte3;
					byte3=byte4;
					byte4=serialPort_.ReadByte();
					//Debug.Log("data is coming" + byte1 + ":"+ byte2 + ":"+ byte3 + ":"+ byte4 );
					if ((byte1 == 1) & (byte2 == 2) & (byte3 == 3) & (byte4 == 4))
					{
						low=serialPort_.ReadByte();
						med=serialPort_.ReadByte();
						high=serialPort_.ReadByte();
						value=(256.0*256.0*high + 256.0*med + low)/nsamples;
						Debug.Log("value"+value);
						//422,1017,0
					}
   
					isNewMessageReceived_ = true; 
				} 
			} catch (System.Exception e) {
				//Debug.LogWarning(e.Message);
			}
		}
	}
	
	public void Write(string message)
	{
		try {
			serialPort_.Write(message);
		} catch (System.Exception e) {
			Debug.LogWarning(e.Message);
		}
	}

	private void createPrefab(Dictionary<string, CustomPrefab1> prefabs, int c, Vector3 p, Vector3 s, string t, GameObject origin)
	{
		string prefabName=t+c;
		prefabs.Add(prefabName, new CustomPrefab1(prefabName, p,s,t,origin));
		prefabs[prefabName].Instantiate();
		prefabs [prefabName].access ().transform.parent = this.gameObject.transform;

	}
	private void destroyPrefab(Dictionary<string, CustomPrefab1> prefabs,string tag)
	{
		for(int i=0;i <prefabs.Count;i++)
		{
			prefabs [tag + i].delete ();
		}
		prefabs.Clear ();
	}

	double map(double s, double a1, double a2, double b1, double b2)
	{
		return b1 + (s-a1)*(b2-b1)/(a2-a1);
	}
}   