﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class ShapeMorphing : MonoBehaviour {


	private List<Vector3> spherePos = new List<Vector3> ();
	private List<Vector3> cubePos = new List<Vector3> ();
	private List<Vector3> torusPos = new List<Vector3> ();
	private List<Vector3> pyramidPos = new List<Vector3> ();
	private int sphereCount = 0;
	private int cubeCount = 0;
	private int voxelCount = 0;
	private int torusCount = 0;

	[SerializeField]
	private float value;

	[SerializeField]
	private GameObject voxelPrefab;

	private Dictionary<string, CustomPrefab1> voxel = new Dictionary<string, CustomPrefab1>();
	[SerializeField]
	private ProceduralTerrain proceduralTerrain;

	[SerializeField]
	private float timeCounter=0.0f;
	[SerializeField]
	private int shapeCounter=1;
	[SerializeField]
	private float timeStep=0.5f;
	[SerializeField]
	private float waitTime=3;
	[SerializeField]
	private bool increasing=true;
	[SerializeField]
	private bool shapeIncreasing=true;

	private float tempSize=50.0f;

	private float waitCounter=0.0f;
	private bool wait = true;

	// Use this for initialization
	void Start () {

		Vector3 center = new Vector3 (0, 0, 0);

		Vector3 tempScale = new Vector3 (1.0f, 1.0f, 1.0f);


		//cube 
		float radius=8;
		//float radius=16;
		for (float i = -radius; i < radius; i++) {
			for (float j = -radius; j < radius; j++) {
				for (float k = -radius; k < radius; k++) {
					Vector3 tempPos = new Vector3 (i, j, k)+center;
					cubePos.Add (tempPos);
					createPrefab (voxel, voxelCount, tempPos, tempScale, "Sphere", voxelPrefab);
					voxelCount++;
					cubeCount++;
				}
			}
		}

		//sphere
		//radius=20;//10;
		radius=10;
		for (float i = -radius; i < radius; i++) {
			for (float j = -radius; j < radius; j++) {
				for (float k = -radius; k < radius; k++) {
					Vector3 tempPos = new Vector3 (i, j, k)+center;
					float distance = Vector3.Distance (center, tempPos);
					if(distance <= radius && sphereCount>=cubeCount)
					{
						//cubePos.Add(center);
					}
					else if (distance <= radius) 
					{
						spherePos.Add (tempPos);
						voxelCount++;
						sphereCount++;
					}
				}
			}
		}

		//torus

		//int fillSize=60;//30;
		//float size = 0.2f;//0.1f;

		//float innerRadius = 12.35f;//6.1f;
		//float thickness = 8.866f;//4.433f;

		int fillSize=30;
		float size = 0.1f;

		float innerRadius = 6.1f;
		float thickness = 4.433f;

		for (int i = -fillSize; i < fillSize; i++) {
			for (int j = -fillSize; j < fillSize; j++) {
				for (int k = -fillSize; k < fillSize; k++) {

					Vector3 cubePosition = new Vector3 (i, j, k);
					Vector3 cubeDirection= new Vector3(cubePosition.x,0,cubePosition.z);
					cubeDirection.Normalize();
					Vector3 donutCenter = cubeDirection * (innerRadius + thickness);

					if (i == 0 && k == 0)
						continue;
					if (Vector3.Distance (cubePosition, donutCenter) < thickness) {
						torusPos.Add (new Vector3(i, j, k));
					}

				}
			}
		}
		while (torusPos.Count < spherePos.Count) {
			torusPos.Add (new Vector3 (0, 0, 0));

		}

		//pyramid
		//float length=21.2f;//10.6f;
		//float maxHeight =27.8f;// 13.9f;

		float length=10.6f;
		float maxHeight =13.9f;
		for (float y = 0; y < maxHeight; y++) {
			length = maxHeight - y;
			for (float x = -length; x <= length; x++) {
				for (float z = -length; z <= length; z++) {
					//if (Mathf.Abs (x) == length || Mathf.Abs (z) == length){ 
					if (Mathf.Abs (x) < length || Mathf.Abs (z) < length) {
						pyramidPos.Add (new Vector3 (x, y, z));
					}
				}
			}
		}
		while (pyramidPos.Count < spherePos.Count) {
			pyramidPos.Add (new Vector3 (0, 0, 0));
			
		}

		/*
		float c= 3.0f;
		float a= 12.0f;
		float step=2.0f*Mathf.PI/spherePos.Count;
		float v = 0.0f; //step


		for (int i = 0; i < spherePos.Count; i++) {

			Vector3 tempVector = new Vector3 ();
			tempVector.x = (c + a * Mathf.Cos (v)) * Mathf.Cos (v);
			tempVector.y = (c + a * Mathf.Cos (v)) * Mathf.Sin (v);
			tempVector.z =  a * Mathf.Sin (v);
			torusPos.Add (tempVector);
			v += step;
			
		}
		//= (c+a */


		Debug.Log (spherePos.Count+","+cubePos.Count+","+torusPos.Count+","+pyramidPos.Count);

		timeCounter = 0.0f;
		shapeCounter = 1;
		
	}
	
	// Update is called once per frame
	void Update () {

		if (waitCounter > waitTime) {
			wait = false;
		}

		if (!wait) {

			if (timeCounter > 10) {
				timeCounter = 0.0F;
				shapeCounter++;
				wait = true;
				waitCounter = 0;
			}
			if (shapeCounter > 4) {
				shapeCounter = 1;
			}


			for (int x = 0; x < proceduralTerrain.size; x++) {
				for (int y = 0; y < proceduralTerrain.size; y++) {
					for (int z = 0; z < proceduralTerrain.size; z++) {
						proceduralTerrain.data [x, y, z] = -1.0f;
					}
				}
			}


			if (shapeCounter == 1) {	

				for (int i = 0; i < cubeCount; i++) {
					GameObject tempGO = GameObject.Find ("Sphere" + i);

					double mappedX = map (timeCounter * 100, 0.0, 1000.0, cubePos [i].x, spherePos [i].x);
					double mappedY = map (timeCounter * 100, 0.0, 1000.0, cubePos [i].y, spherePos [i].y);
					double mappedZ = map (timeCounter * 100, 0.0, 1000.0, cubePos [i].z, spherePos [i].z);

					tempGO.transform.position = new Vector3 ((float)mappedX, (float)mappedY, (float)mappedZ);

					mappedX = map (mappedX, -tempSize, tempSize, 1.0, proceduralTerrain.size - 2.0);
					mappedY = map (mappedY, -tempSize, tempSize, 1.0, proceduralTerrain.size - 2.0);
					mappedZ = map (mappedZ, -tempSize, tempSize, 1.0, proceduralTerrain.size - 2.0);

					proceduralTerrain.data [(int)(mappedX), (int)(mappedY), (int)(mappedZ)] = 0.9f;
				}
			} else if (shapeCounter == 2) {
				for (int i = 0; i < cubeCount; i++) {
					GameObject tempGO = GameObject.Find ("Sphere" + i);

					double mappedX = map (timeCounter * 100, 0.0, 1000.0, spherePos [i].x, torusPos [i].x);
					double mappedY = map (timeCounter * 100, 0.0, 1000.0, spherePos [i].y, torusPos [i].y);
					double mappedZ = map (timeCounter * 100, 0.0, 1000.0, spherePos [i].z, torusPos [i].z);

					tempGO.transform.position = new Vector3 ((float)mappedX, (float)mappedY, (float)mappedZ);

					mappedX = map (mappedX, -tempSize, tempSize, 1.0, proceduralTerrain.size - 2.0);
					mappedY = map (mappedY, -tempSize, tempSize, 1.0, proceduralTerrain.size - 2.0);
					mappedZ = map (mappedZ, -tempSize, tempSize, 1.0, proceduralTerrain.size - 2.0);

					proceduralTerrain.data [(int)(mappedX), (int)(mappedY), (int)(mappedZ)] = 0.9f;
				}
			} else if (shapeCounter == 3) {
				for (int i = 0; i < cubeCount; i++) {
					GameObject tempGO = GameObject.Find ("Sphere" + i);

					double mappedX = map (timeCounter * 100, 0.0, 1000.0, torusPos [i].x, pyramidPos [i].x);
					double mappedY = map (timeCounter * 100, 0.0, 1000.0, torusPos [i].y, pyramidPos [i].y);
					double mappedZ = map (timeCounter * 100, 0.0, 1000.0, torusPos [i].z, pyramidPos [i].z);

					tempGO.transform.position = new Vector3 ((float)mappedX, (float)mappedY, (float)mappedZ);

					mappedX = map (mappedX, -tempSize, tempSize, 1.0, proceduralTerrain.size - 2.0);
					mappedY = map (mappedY, -tempSize, tempSize, 1.0, proceduralTerrain.size - 2.0);
					mappedZ = map (mappedZ, -tempSize, tempSize, 1.0, proceduralTerrain.size - 2.0);

					proceduralTerrain.data [(int)(mappedX), (int)(mappedY), (int)(mappedZ)] = 0.9f;
				}
			} else if (shapeCounter == 4) {
				for (int i = 0; i < cubeCount; i++) {
					GameObject tempGO = GameObject.Find ("Sphere" + i);

					double mappedX = map (timeCounter * 100, 0.0, 1000.0, pyramidPos [i].x, cubePos [i].x);
					double mappedY = map (timeCounter * 100, 0.0, 1000.0, pyramidPos [i].y, cubePos [i].y);
					double mappedZ = map (timeCounter * 100, 0.0, 1000.0, pyramidPos [i].z, cubePos [i].z);

					tempGO.transform.position = new Vector3 ((float)mappedX, (float)mappedY, (float)mappedZ);

					mappedX = map (mappedX, -tempSize, tempSize, 1.0, proceduralTerrain.size - 2.0);
					mappedY = map (mappedY, -tempSize, tempSize, 1.0, proceduralTerrain.size - 2.0);
					mappedZ = map (mappedZ, -tempSize, tempSize, 1.0, proceduralTerrain.size - 2.0);


					proceduralTerrain.data [(int)(mappedX), (int)(mappedY), (int)(mappedZ)] = 0.9f;
				}
			}

			proceduralTerrain.ApplyDataToMesh ();

			timeCounter += Time.deltaTime * timeStep;
		} else {
			waitCounter += Time.deltaTime;
		}
		
	}

	private void createPrefab(Dictionary<string, CustomPrefab1> prefabs, int c, Vector3 p, Vector3 s, string t, GameObject origin)
	{
		string prefabName=t+c;
		prefabs.Add(prefabName, new CustomPrefab1(prefabName, p,s,t,origin));
		prefabs[prefabName].Instantiate();
		prefabs [prefabName].access ().transform.parent = this.gameObject.transform;

	}
	private void destroyPrefab(Dictionary<string, CustomPrefab1> prefabs,string tag)
	{
		for(int i=0;i <prefabs.Count;i++)
		{
			prefabs [tag + i].delete ();
		}
		prefabs.Clear ();
	}

	double map(double s, double a1, double a2, double b1, double b2)
	{
		return b1 + (s-a1)*(b2-b1)/(a2-a1);
	}
}
