﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

[RequireComponent (typeof (MeshFilter))]
[RequireComponent (typeof (MeshRenderer))]
[AddComponentMenu("Mesh/Dynamic Mesh Generator")]
public class DynamicMesh: MonoBehaviour {

	//Clockwise winding order defines the front of the vertex

	private int srx = 41;
    private int sry = 41;
	private double meshHeight= 0.0;

	private double x0 = -2;
    private double x1 = 2;
    private double y0 = -2;
    private double y1 = 2;

	Mesh vizmesh;
    Vector3[] vizp; 
    int[] viztri;
	

	bool regenerateCustom = false;
	void Start(){
        
		setupMesh();
		flipMeshNormals(this.gameObject);
		renderMeshes();

	}
	
	// Update is called once per frame
	void Update () {

	for (int j = 0; j < sry; ++j)
	{
		for (int i = 0; i < srx; ++i)
		{
			Vector3 s= new Vector3((float)x0,(float)meshHeight,(float)y0);
			float dist = Vector3.Distance(vizp[j * srx + i],s);
			vizp[j * srx + i].y+= Mathf.Sin((float)Time.time+dist*1.0f)* 0.005f;
		}
	}
	renderMeshes();
	}


	#region MeshFunctions
	
	void setupMesh(){
		vizp = new Vector3[srx*sry];
        viztri = new int[(srx - 1) * (sry - 1) * 6];

		double dx = (x1 - x0) / (srx - 1.0);
		double dy = (y1 - y0) / (sry - 1.0);

		for (int j = 0; j < sry; ++j)
        {
            for (int i = 0; i < srx; ++i)
            {
                double x = x0 + i * dx;
                double y = y0 + j * dy;

				vizp[j * srx + i] = new Vector3((float)x, (float)meshHeight, (float)y);

				//vizuv[j * srx + i] = new Vector2( map(3.0f,2.0f,4.0f,0.0f,1.0f), 0.0f);
            }
        }
		int k = 0;
        for (int j = 0; j < sry - 1; ++j)
        {
            for (int i = 0; i < srx - 1; ++i)
            {
                {
                    int n0 = j * srx + i;

                    //DRAW THE MESHES
                    viztri[k++] = n0;
                    viztri[k++] = n0+1+srx;
                    viztri[k++] = n0+1;

                    viztri[k++] = n0;
                    viztri[k++] = n0  +srx;
                    viztri[k++] = n0 + srx+1;
                }
            }
        }

		vizmesh = new Mesh();
        vizmesh.vertices = vizp;
        vizmesh.triangles = viztri;
        MeshFilter mf = GetComponent<MeshFilter>();
        mf.mesh = vizmesh;

	}
	void renderMeshes()
    {
        vizmesh.vertices = vizp;

        vizmesh.RecalculateNormals();
        vizmesh.RecalculateBounds();

        GetComponent<MeshFilter>().mesh = vizmesh;
        Debug.Log("Mesh reassigned");

    }
	void flipMeshNormals(GameObject g)
    {
        MeshFilter filter = g.GetComponent(typeof(MeshFilter)) as MeshFilter;
        if (filter != null)
        {
            Mesh mesh = filter.mesh;

            Vector3[] normals = mesh.normals;
            for (int i = 0; i < normals.Length; i++)
                normals[i] = -normals[i];
            mesh.normals = normals;

            for (int m = 0; m < mesh.subMeshCount; m++)
            {
                int[] triangles = mesh.GetTriangles(m);
                for (int i = 0; i < triangles.Length; i += 3)
                {
                    int temp = triangles[i + 0];
                    triangles[i + 0] = triangles[i + 1];
                    triangles[i + 1] = temp;
                }
                mesh.SetTriangles(triangles, m);
            }
        }
    }

    void changeVertixMeshColor()
    {
        Mesh mesh = GetComponent<MeshFilter>().mesh;
        Vector3[] vertices = mesh.vertices;

        // create new colors array where the colors will be created.
        UnityEngine.Color[] colors = new UnityEngine.Color[vertices.Length];

		for (int i = 0; i < vertices.Length; i++) {
			colors [i] = UnityEngine.Color.Lerp (UnityEngine.Color.red, UnityEngine.Color.blue, vertices [i].y);
		}
        // assign the array of colors to the Mesh.
        mesh.colors = colors;
    }
    ///////
    #endregion

}
